#include <msp/gl/font.h>
#include "dropdown.h"
#include "list.h"
#include "panel.h"
#include "part.h"
#include "root.h"
#include "style.h"
#include "text.h"

using namespace std;

namespace Msp {
namespace GLtk {

Dropdown::Dropdown()
{
	init();
}

Dropdown::Dropdown(ListData &d):
	list(d)
{
	init();
}

void Dropdown::init()
{
	dropped = false;

	add(list);
	list.set_view_all();
	list.signal_item_selected.connect(sigc::mem_fun(this, &Dropdown::list_item_selected));
	list.signal_autosize_changed.connect(sigc::mem_fun(this, &Dropdown::list_autosize_changed));
}

void Dropdown::autosize_special(const Part &part, Geometry &ageom)
{
	if(part.get_name()=="list")
	{
		list.autosize();
		ageom.w = max(ageom.w, list.get_geometry().w);
	}
	else if(part.get_name()=="text")
	{
		const Sides &margin = part.get_margin();
		const GL::Font &font = style->get_font();
		float font_size = style->get_font_size();

		unsigned max_w = 0;
		const ListData &data = list.get_data();
		for(unsigned i=0; i<data.size(); ++i)
		{
			unsigned w = static_cast<unsigned>(font.get_string_width(data.get_string(i))*font_size);
			max_w = max(max_w, w);
		}
		ageom.w = max(ageom.w, max_w+margin.left+margin.right);

		unsigned line_height = static_cast<unsigned>((font.get_ascent()-font.get_descent())*font_size);
		ageom.h = max(ageom.h, line_height+margin.top+margin.bottom);
	}

	rebuild();
}

void Dropdown::set_selected_index(int index)
{
	list.set_selected_index(index);
	if(index<0)
		text.set(string());
}

void Dropdown::rebuild_special(const Part &part)
{
	if(part.get_name()=="text")
		text.build(part, geom, part_cache);
	else
		Widget::rebuild_special(part);
}

void Dropdown::render_special(const Part &part, GL::Renderer &renderer) const
{
	if(part.get_name()=="list" && dropped)
		list.render(renderer);
}

void Dropdown::button_press(int x, int y, unsigned btn)
{
	if(dropped)
	{
		Container::button_press(x, y, btn);
		if(!click_focus)
		{
			dropped = false;
			clear_state(ACTIVE);
			signal_ungrab_pointer.emit();
		}
	}
	else if(btn==1)
	{
		dropped = true;
		set_state(ACTIVE);
		signal_grab_pointer.emit();
	}
}

void Dropdown::on_geometry_change()
{
	resize_list();
}

void Dropdown::on_style_change()
{
	text.set_style(style);
	resize_list();
}

void Dropdown::list_autosize_changed()
{
	resize_list();
	signal_autosize_changed.emit();
}

void Dropdown::resize_list()
{
	list.autosize();
	Geometry lgeom = list.get_geometry();
	lgeom.x = 0;
	lgeom.y = -lgeom.h;
	lgeom.w = max(geom.w, lgeom.w);
	int root_x = geom.x;
	int root_y = geom.y;
	for(Widget *p=parent; p; p=p->get_parent())
	{
		root_x += p->get_geometry().x;
		root_y += p->get_geometry().y;
		if(Root *root = dynamic_cast<Root *>(p))
		{
			const Geometry &rgeom = root->get_geometry();
			if(lgeom.h*2>rgeom.h)
				lgeom.h = rgeom.h/2;
			if(root_y+lgeom.y<0)
				lgeom.y = -root_y;
			if(root_y+lgeom.y+lgeom.h>rgeom.h)
				lgeom.y = rgeom.h-lgeom.h-root_y;
			if(root_x+lgeom.x+lgeom.w>rgeom.w)
				lgeom.x = rgeom.w-lgeom.w-root_x;
			break;
		}
	}
	list.set_geometry(lgeom);
}

void Dropdown::list_item_selected(unsigned index)
{
	if(dropped)
	{
		dropped = false;
		clear_state(ACTIVE);
		signal_ungrab_pointer.emit();
	}

	text.set(list.get_data().get_string(index));

	signal_item_selected.emit(index);
	rebuild();
}


Dropdown::Loader::Loader(Dropdown &d):
	DataFile::DerivedObjectLoader<Dropdown, Widget::Loader>(d)
{
	add("item", &Loader::item);
}

void Dropdown::Loader::item(const string &v)
{
	dynamic_cast<BasicListData<string> &>(obj.list.get_data()).append(v);
}

} // namespace GLtk
} // namespace Msp
